@@ -39,13 +39,32 @@ class Device(EventListener):
     def is_dishwasher(self) -> bool:
         return False
 
+    def __is_success(self, status_code: int) -> bool:
+        return status_code >= 200 and status_code <= 299
+
     def _perform_get(self, path:str) -> Dict[str, Any]:
         uri = self._uri + path
         logging.info("query GET " + uri)
         response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token})
-        response.raise_for_status()
-        data = response.json()
-        return data
+        if self.__is_success(response.status_code):
+            return response.json()
+        else:
+            logging.warning("error occurred by calling GET " + uri)
+            logging.warning("got " + str(response.status_code) + " " + response.text)
+            raise Exception("error occurred by calling GET " + uri + " Got " + str(response))
+
+    def _perform_put(self, path:str, data: str, num_tries: int = 1):
+        uri = self._uri + path
+        for i in range(0, num_tries):
+            logging.info("query PUT " + uri)
+            response = requests.put(uri, data=data, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token})
+            if self.__is_success(response.status_code):
+                return
+            else:
+                logging.warning("error occurred by calling POST " + uri + " " + data)
+                logging.warning("got " + response.text)
+                sleep(2)
+        raise Exception("error occurred by calling GET " + uri + " Got " + str(response.status_code) + " " + str(response.text))
 
     @property
     def __fingerprint(self) -> str:
@@ -77,9 +96,10 @@ class Dishwasher(Device):
         self.__door = ""
         self.__program_selected = ""
         self.remote_start_allowed = False
-        self.__program_start_in_relative_sec = ""
+        self.__program_start_in_relative_sec = 0
         self.__program_remaining_time = ""
-        self.program_progress = 100
+        self.__program_progress = 0
+        self.__program_active = ""
         self.__program_remote_control_active = ""
         self.program_extra_try = ""
         self.program_hygiene_plus = ""
@@ -103,8 +123,6 @@ class Dishwasher(Device):
     def on__value_changed_event(self, event):
         if event.id == self.haid:
             try:
-                logging.info("event received: " + str(event))
-                logging.info("data: " + str(event.data))
                 data = json.loads(event.data)
                 self.__on_value_changes(data.get('items', []))
                 for value_changed_listener in self._value_changed_listeners:
@@ -122,12 +140,16 @@ class Dishwasher(Device):
                 self.remote_start_allowed = record['value']
             elif record['key'] == 'BSH.Common.Setting.PowerState':
                 self.__power = record['value']
+            elif record['key'] == 'BSH.Common.Root.SelectedProgram':
+                self.__program_selected = record['value']
+            elif record['key'] ==  'BSH.Common.Root.ActiveProgram':
+                self.__program_active = record['value']
             elif record['key'] == 'BSH.Common.Option.StartInRelative':
                 self.__program_start_in_relative_sec = record['value']
             elif record['key'] == 'BSH.Common.Option.RemainingProgramTime':
                 self.__program_remaining_time = record['value']
             elif record['key'] == 'BSH.Common.Option.ProgramProgress':
-                self.program_progress = record['value']
+                self.__program_progress = record['value']
             elif record['key'] == 'BSH.Common.Status.RemoteControlActive':
                 self.__program_remote_control_active = record['value']
             elif record['key'] == 'Dishcare.Dishwasher.Option.ExtraDry':
@@ -137,7 +159,7 @@ class Dishwasher(Device):
             elif record['key'] == 'Dishcare.Dishwasher.Option.VarioSpeedPlus':
                 self.program_vario_speed_plus = record['value']
             else:
-                print(record)
+                logging.info("unknown changed " + str(record))
 
     def on_keep_alive_event(self, event):
         if datetime.now() > (self.date_refreshed + timedelta(minutes=45)):
@@ -145,17 +167,14 @@ class Dishwasher(Device):
             self.__refresh()
 
     def __refresh(self):
-        settings = self._perform_get('/settings')['data']['settings']
-        self.__on_value_changes(settings)
-
-        status = self._perform_get('/status')['data']['status']
-        self.__on_value_changes(status)
-
+        self.__on_value_changes(self._perform_get('/settings')['data']['settings'])
+        self.__on_value_changes(self._perform_get('/status')['data']['status'])
         record = self._perform_get('/programs/selected')['data']
         self.__program_selected = record['key']
         self.__on_value_changes(record['options'])
-
         self.date_refreshed = datetime.now()
+        for value_changed_listener in self._value_changed_listeners:
+            value_changed_listener()
 
     @property
     def power(self):
@@ -169,6 +188,13 @@ class Dishwasher(Device):
     def operation(self):
         return self.__operation[self.__operation.rindex('.')+1:]
 
+    @property
+    def program_progress(self):
+        if self.operation.lower() == 'run':
+            return self.__program_progress
+        else:
+            return 0
+
     @property
     def program_selected(self):
         return self.__program_selected[self.__program_selected.rindex('.')+1:]
@@ -181,13 +207,24 @@ class Dishwasher(Device):
         else:
             return ""
 
-    @start_date.setter
-    def start_date(self, dt: str):
+    def to_period(self, future_date: str):
+        remaining_secs_to_wait = (datetime.fromisoformat(future_date) - datetime.now()).seconds
+        if remaining_secs_to_wait > (60 * 60):
+            return int(remaining_secs_to_wait / 60), "minutes"
+        else:
+            return remaining_secs_to_wait, "seconds"
+
+
+    def set_start_date(self, dt: str):
         if self.operation == "BSH.Common.EnumType.OperationState.Run":
             logging.info("dishwasher is already running")
         else:
-            remaining_secs_to_wait = (datetime.fromisoformat(dt) - datetime.now()).seconds
-            uri = self._uri + "/programs/active"
+            dz = datetime.now()
+            delay = (datetime.fromisoformat(dt) - datetime.now())
+            remaining_secs_to_wait = int((datetime.fromisoformat(dt) - datetime.now()).total_seconds())
+            if remaining_secs_to_wait > 86000:
+                logging.warning("large delay " + str(remaining_secs_to_wait) + " (start date: " + dt + ") reduced to 86000")
+                remaining_secs_to_wait = 86000
             data = {
                 "data": {
                     "key": self.__program_selected,
@@ -198,10 +235,8 @@ class Dishwasher(Device):
                                 } ]
                 }
             }
-            logging.info("query PUT " + uri)
-            js = json.dumps(data)
-            response = requests.put(uri, data=js, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token})
-            response.raise_for_status()
+            self._perform_put("/programs/active", json.dumps(data, indent=2), num_tries=3)
+            logging.info("dishwasher program " + self.program_selected + " starts in " + str(remaining_secs_to_wait) + " secs")
             self.__refresh()
 
     def __str__(self):
@@ -261,7 +296,7 @@ class HomeConnect:
                         for notify_listener in self.notify_listeners:
                             notify_listener.on_status_event(event)
                     else:
-                        print(event)
+                        logging.info("unknown event type " + str(event.event))
             except Exception as e:
                 logging.warning("Error occurred by opening sse socket to " + uri + " " + str(e))
                 wait_time_sec = {0: 3, 1:5, 2: 30, 3: 2*60, 4: 5*60}.get(num_reconnect, 30*60)
