@@ -197,7 +197,7 @@ class DishwasherThing(Thing):
                          'readOnly': True,
                      }))
 
-        self.start_date = Value(dishwasher.start_date, dishwasher.start_date)
+        self.start_date = Value(dishwasher.start_date, dishwasher.set_start_date)
         self.add_property(
             Property(self,
                      'program_start_date',
@@ -205,8 +205,8 @@ class DishwasherThing(Thing):
                      metadata={
                          'title': 'Start date',
                          "type": "string",
-                         'description': 'Th start date',
-                         'readOnly': True,
+                         'description': 'The start date',
+                         'readOnly': False,
                      }))
 
         self.program_progress = Value(dishwasher.program_progress)
